package infomat.evaluation;

import java.util.HashMap;
import java.util.Vector;
import java.io.*;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;


/**
 * For calculating intrinsic quality measures.
 * 
 * @version          080608
 * @author           Magnus Rosell
 */
public class MatrixMeasures{

    private Properties properties;

    private IMatrix matrix;
    private IObjectGrouping evalGrouping;
    private IObjectGrouping refGrouping;
    private String evalGroupingDesignation;
    private String refGroupingDesignation;
    private String stamp;

    public MatrixMeasures(IMatrix matrix, 
			  IObjectGrouping evalGrouping, 
			  IObjectGrouping refGrouping, 
			  String evalGroupingDesignation, 
			  String refGroupingDesignation, 
			  String stamp){
	this.matrix = matrix;
	this.evalGrouping = evalGrouping;
	this.refGrouping = refGrouping;
	this.evalGroupingDesignation = evalGroupingDesignation;
	this.refGroupingDesignation = refGroupingDesignation;
	this.stamp=stamp;

	this.properties=getDefaultProperties();

    }

    public Properties getDefaultProperties(){

	Properties properties = 
	    new Properties("Matrix Measures Properties", CurrentClassGetter.getClassName());
		
	    /*
	try{
	    Property p = new Property("Group Evaluation", true, "Evaluate groups individually");
	    PropertyValueDescription pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	    */
	return properties;
    }

    public Measures calculateAllMeasures() 
	throws MeasureNameConflictException{
	Measures measures = new Measures("Matrix Measures");

	measures.addMeasure(calculateBasicMatrixMeasures());

	// Basic Grouping Measures
	if(evalGrouping!=null)
	    measures.addMeasure(calculateGroupingMeasures(evalGrouping, evalGroupingDesignation));
	if(refGrouping!=null)
	    measures.addMeasure(calculateGroupingMeasures(refGrouping, refGroupingDesignation));

	/*
	// Grouping Information Measures
	if(evalGrouping!=null)
	    measures.addMeasure(calculateGroupingInformationMeasures(evalGrouping, evalGroupingDesignation, refGrouping));
	if(refGrouping!=null)
	    measures.addMeasure(calculateGroupingMeasures(refGrouping, refGroupingDesignation, evalGrouping));
	*/

	measures.setAllMeasuresStampsRecursively(stamp);

	return measures;
    }


    public MeasureGroup allMeasuresInAGroup() 
	throws MeasureNameConflictException{

	MeasureGroup mg = new MeasureGroup("Matrix Measures");
	mg.addMeasure(calculateBasicMatrixMeasures());

	// Basic Grouping Measures
	if(evalGrouping!=null)
	    mg.addMeasure(calculateGroupingMeasures(evalGrouping, evalGroupingDesignation));
	if(refGrouping!=null)
	    mg.addMeasure(calculateGroupingMeasures(refGrouping, refGroupingDesignation));

	/*
	// Grouping Information Measures
	if(evalGrouping!=null)
	    measures.addMeasure(calculateGroupingInformationMeasures(evalGrouping, evalGroupingDesignation, refGrouping));
	if(refGrouping!=null)
	    measures.addMeasure(calculateGroupingMeasures(refGrouping, refGroupingDesignation, evalGrouping));
	*/

	mg.setAllMeasuresStampsRecursively(stamp);

	return mg;
    }

    /*
    // "Evaluated Grouping"
    // "Reference Grouping"
    public MeasureGroup calculateGroupingInformationMeasures(IObjectGrouping grouping, String groupingDesignation, IObjectGrouping otherGrouping) throws MeasureNameConflictException{

	MeasureGroup mg = new MeasureGroup(groupingDesignation+" Information Measures");
	Measure m = new Measure("Entropy Of IObjectGroups", entropyForIObjectGroups(grouping), "Entropy for distribution IObjects over IObjectGroups in IObjectGrouping");
	mg.addMeasure(m);
	
	m = new Measure("Entropy Of IObjectGroups", entropyForIObjectGroups(grouping), "Entropy for distribution IObjects over IObjectGroups in IObjectGrouping");
	mg.addMeasure(m);

	return mg;
    }
    */

    // "Evaluated Grouping"
    // "Reference Grouping"
    public MeasureGroup calculateGroupingMeasures(IObjectGrouping grouping, String groupingDesignation) throws MeasureNameConflictException{

	MeasureGroup mg = new MeasureGroup(groupingDesignation);
	Measure m = new Measure("Number Of IObjects", grouping.numberOfIObjects(), "Number Of IObjects in IObjectGrouping");
	mg.addMeasure(m);

	double cellsPerObj = avgCellsPerObjInGroup(grouping);
	m = new Measure("Cells per IObject", cellsPerObj, "Number Of IMatrixCells Per IObject in the IObjectGrouping");
	mg.addMeasure(m);

	m = new Measure("Number Of IObjectGroups", grouping.numberOfIObjectGroups(), "Number Of IObjectGroups in IObjectGrouping");
	mg.addMeasure(m);

	double entropy = 
	    ExtrinsicEvaluation.entropyForIObjectGroups(grouping);
	m = new Measure("Entropy Of IObjectGroups", entropy, "Entropy for distribution IObjects over IObjectGroups in IObjectGrouping");
	mg.addMeasure(m);

	double normEntropy = 
	    entropy/MoreMath.log2(grouping.numberOfIObjectGroups());
	m = new Measure("Norm Entropy of IObjectGroups", normEntropy, "Norm Entropy of IObjectGroups ( E(G)/log(|G|) )");
	mg.addMeasure(m);



	return mg;
    }


    public MeasureGroup calculateBasicMatrixMeasures() throws MeasureNameConflictException{

	MeasureGroup mg = new MeasureGroup("Basic Matrix Measures");
	Measure m = new Measure("Number Of Row IObjects", matrix.getRowIObjectSet().numberOfIObjects(), "Number Of Row IObjects in IMatrix");
	mg.addMeasure(m);

	m = new Measure("Number Of Column IObjects", matrix.getColumnIObjectSet().numberOfIObjects(), "Number Of Column IObjects in IMatrix");
	mg.addMeasure(m);

	double cellsPerRow = avgCellsPerObjInGroup(matrix.getRowIObjectSet());
	m = new Measure("Cells per Row", cellsPerRow, "Average Number of IMatrixCells Per Row IObjects in IMatrix.");
	mg.addMeasure(m);

	double cellsPerCol = avgCellsPerObjInGroup(matrix.getColumnIObjectSet());
	m = new Measure("Cells per Column", cellsPerCol, "Average Number of IMatrixCells Per Column IObjects in IMatrix.");
	mg.addMeasure(m);


	return mg;
    }

    private double avgCellsPerObjInGroup(IObjectGroup iog){
	double avg = 0.0;
	Vector<IObject> ioVec = iog.getOrdered();
	//	Vector<IObject> badOnes = new Vector<IObject>();
	for(int i=0; i<ioVec.size(); i++){
	    Vector cells = matrix.getCellsWith(ioVec.elementAt(i));

	    /*
	    if(cells==null){
		IObject io = ioVec.elementAt(i);
		System.out.println("String: "+io.getString());
		System.out.println("ID: "+io.getID());
		System.out.println("Kind: "+io.getKind());
		if(io instanceof IObjectGrouping)
		    System.out.println("IObjectGrouping!");
		else if(io instanceof ILabel)
		    System.out.println("ILabel!");
		else if(io instanceof IObjectGroup)
		    System.out.println("IObjectGroup!");
		System.out.println("Location: "+io.getLocation());
		System.out.println();
		badOnes.add(io);
	    }else
	    */
	    
	    //	    if(cells=!null)
		avg += cells.size();
		//	    printBadOnes(iog, badOnes);
	}
	avg /= ioVec.size();
	return avg;
    }

    /*
    private void printBadOnes(IObjectGroup iog, Vector<IObject> badOnes){
	String file = "/var/tmp/rosell/badOnes/"+iog.getName();

	try{
	PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));
	for(int i=0; i<badOnes.size(); i++){
	    pw.println(badOnes.elementAt(i).getLocation());
	}
	pw.close();
	}catch(java.io.IOException ioe){
	    // So sue me
	}
    }
    */


    //"Entropy for distribution IObjects over IObjectGroups in IObjectGrouping"
    /*
    public double entropyForIObjectGroups(IObjectGrouping grouping){
	double entropy = 0.0;
	int size = grouping.numberOfIObjects();
	Vector<IObjectGroup> groups = grouping.getGroups();
	for(int i=0; i<groups.size(); i++){
	    double count = groups.get(i).numberOfIObjects();
	    if(count>0){
		double p = count/size;
		entropy -= p*MoreMath.log2(p);
	    }
	}
	return entropy;
    }
    */

    /************************************/

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public Properties getProperties(){
    	return properties;
    }

    public void setProperties(Properties properties){
    	this.properties=properties;
    }

}
