package infomat.vectorspace;

import infomat.vectorspace.vector.*;
import infomat.io.*;
import mro.util.*;
import mro.util.doublehashmap.DoubleHashMap;

import java.io.*;
import java.util.*;
import java.awt.Dimension;

import javax.swing.JLabel;


/**
 * IMatrix represents the relations between two IObjectSet:s.
 * The IMatrixCell class represents the elements of this IMatrix. 
 * This is implemented considering sparse matrices. The IMatrixCells
 * are stored in a DoubleHashMap and are the non zero matrix elements.
 *
 * @param rows A IObjectSet
 * @param cols A IObjectSet
 * @param cellMap A DoubleHashMap that stores the non zero elements of the matrix.
 *
 * @version          081031
 * @author           Magnus Rosell
 */
public class IMatrix{
    private IObjectSet rows;
    private IObjectSet cols;
    private DoubleHashMap cellMap;

    //    static private float maxW; // XXX ugly, warning - this should be fixed

    private HashMap<IObject, Float> rowIObjectGlobalWeightMap;
    private HashMap<IObject, Float> colIObjectGlobalWeightMap;


    public IMatrix(String columnLabel, String rowLabel){
	rows = new IObjectSet(rowLabel); 
	cols = new IObjectSet(columnLabel); 
	cellMap = new DoubleHashMap();
	rowIObjectGlobalWeightMap = new HashMap<IObject, Float>();
	colIObjectGlobalWeightMap = new HashMap<IObject, Float>();
	//	maxW = 0;
    }

    /**
     * Copy constructor. Makes new instances of everything, 
     * but the IObject:s, which are the same.
     */
    public IMatrix(IMatrix m){

	rows = new IObjectSet(m.getRowIObjectSet());
	cols = new IObjectSet(m.getColumnIObjectSet());
	cellMap = new DoubleHashMap();
	rowIObjectGlobalWeightMap = new HashMap<IObject, Float>();
	colIObjectGlobalWeightMap = new HashMap<IObject, Float>();
	//	maxW = 0;


	//	// cols
	//	Vector<IObject> ios = m.cols.getOrdered();
	//	for(int i=0; i<ios.size(); i++)
	//	    addColumnIObject(ios.elementAt(i));
	
	//	// rows
	//	ios = m.rows.getOrdered();
	//	for(int i=0; i<ios.size(); i++)
	//	    addRowIObject(ios.elementAt(i));

	// cells
	Vector cells = m.getAllCells();
	for(int i=0; i<cells.size(); i++)
	    addCell(new IMatrixCell((IMatrixCell) cells.elementAt(i)));

	// global weight
	Iterator<Map.Entry<IObject, Float>> it = 
	    m.rowIObjectGlobalWeightMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<IObject, Float> entry = it.next();
	    setRowIObjectGlobalWeight(entry.getKey(), entry.getValue());
	}

	it = m.colIObjectGlobalWeightMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<IObject, Float> entry = it.next();
	    setColumnIObjectGlobalWeight(entry.getKey(), entry.getValue());
	}
    }

    /**
     * Returns a copy of the specified portion of the matrix.
     * The IMatrixCell:s are copied, but the IObject:s are the same.
     * The global weights of the IObject:s are not kept.
     */
    public IMatrix subMatrix(IObjectGroup rowGroup, IObjectGroup colGroup){
	/*
	if(rowObject(colGroup)){
	    IObjectGroup tmp = rowGroup;
	    rowGroup=colGroup;
	    colGroup=rmpGroup;
	}
	*/

	IMatrix retMat = new IMatrix(rows.getName(), cols.getName());
	Vector<IObject> ios = colGroup.getOrdered();
	for(int i=0; i<ios.size(); i++){
	    retMat.addColumnIObject(ios.elementAt(i));
	}
	ios = rowGroup.getOrdered();
	for(int i=0; i<ios.size(); i++){
	    retMat.addRowIObject(ios.elementAt(i));
	}

	for(int i=0; i<ios.size(); i++){
	    IObject rowObj = ios.elementAt(i);
	    Vector cells =  
		getCellsWithIObjectRestrictedToGroup(rowObj, colGroup);
	    for(int j=0; j<cells.size(); j++){
		IMatrixCell newMC = 
		    new IMatrixCell((IMatrixCell) cells.elementAt(j));
		retMat.addCell(newMC);
	    }
	}

	// global weight
	Iterator<Map.Entry<IObject, Float>> it = 
	    rowIObjectGlobalWeightMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<IObject, Float> entry = it.next();
	    retMat.setRowIObjectGlobalWeight(entry.getKey(), entry.getValue());
	}

	it = colIObjectGlobalWeightMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<IObject, Float> entry = it.next();
	    retMat.setColumnIObjectGlobalWeight(entry.getKey(), entry.getValue());
	}


	return retMat;
    }




    /**
     * adds a Word as Column to the matrix
     * @param w  The Word as a string
     * @param ws  A unique identifier of the word
     */
    /*
    public void addWord(String w, int ws)
    {
	IObject cob = new IObject(w, ws); 
	cols.add(cob);
    }
    */

    /**
     * adds a Word as a Column IObject to the matrix
     * @param w  The Word as a string
     * @param ws  A unique identifier of the word
     * @param globalWeight  The global weight of the word
     */
    /*
    public void addWord(String w, int ws, float globalWeight)
    {
	IObject cob = new IObject(w, ws); 
	objectGlobalWeightMap.put(cob, globalWeight);
	cols.add(cob);
    }
    */

    /**
     * Adds a Column IObject to the matrix.
     * OBS. If you add the same object again it will be added, which
     * may lead to confusion.
     */
    public void addColumnIObject(IObject colObj){
	cols.add(colObj);
    }


    /**
     * adds a Document as a Row to the matrix
     * @param d  The source of the document
     * @param di  A unique identifier of the document
     */
    /*
    public void addDocument(String d, int di)
    {
	IObject cob = new IObject(d, di); //, this); //4 - Document
	rows.add(cob);
    }
    */

    /**
     * adds a Document as a Row to the matrix
     * @param d  The source of the document
     * @param f The location of the document
     * @param di  A unique identifier of the document
     * @see IObject
     */
    /*
    public void addDocument(String d, String f, int di)
    {
	IObject cob = new IObject(d, f, di); //, this); //4 - Document
	rows.add(cob);
    }
    */

    /**
     * Adds a Row IObject to the matrix.
     * OBS. If you add the same object again it will be added, which
     * may lead to confusion.
     */
    public void addRowIObject(IObject rowObj){
	rows.add(rowObj);
    }


    /**
     * Add a IMatrixCell.
     * @param mc the Matrix cell
     * @return true if the corresponding IObjects exist in the IObjectSet:s, false otherwise.
     */
    public boolean addCell(IMatrixCell mc){
	IObject rowObject = 
	    rows.getIObjectWithId(mc.getRowIObject().getID());
	IObject colObject = 
	    cols.getIObjectWithId(mc.getColumnIObject().getID());
	if(rowObject==null || colObject==null)
	    return false;
	cellMap.put(rowObject, colObject, mc);

	return true;
    }
	
    /**
     * Add the value of a column in a row
     * @param ri  The unique identifier of the row
     * @param ci  The unique identifier of the column
     * @param weight  The value of the column in the row
     * @return true if row and column exists, false otherwise
     */
    public boolean addCell(int ri, int ci, float weight)
    {
	IObject rowObject = rows.getIObjectWithId(ri);
	IObject colObject = cols.getIObjectWithId(ci);
	if(rowObject==null || colObject==null)
	    return false;
	IMatrixCell cell = 
	    new IMatrixCell(rowObject, colObject, weight);
	if(!addCell(cell))
	    return false;
	//cellMap.put(rowObject, colObject, cell);

	//	maxW = Math.max(maxW,weight);
	return true;
    }

    /**
     * Add the value of a column in a row
     * @param ri  The unique identifier of the row
     * @param ci  The unique identifier of the column
     * @param weight  The value of the column in the row
     * @return true if row and column exists, false otherwise
     */
    public boolean addCell(int ri, int ci, float localCount, float weight)
    {
	IObject rowObject = rows.getIObjectWithId(ri);
	IObject colObject = cols.getIObjectWithId(ci);
	if(rowObject==null || colObject==null)
	    return false;
	IMatrixCell cell = 
	    new IMatrixCell(rowObject, colObject, localCount, weight);
	if(!addCell(cell))
	    return false;
	//cellMap.put(rowObject, colObject, cell);

	//	maxW = Math.max(maxW,weight);
	return true;
    }

    /**
     * Add the value of a column in a row
     * @param ri  The unique identifier of the row
     * @param ci  The unique identifier of the column
     * @param weight  The value of the column in the row
     * @return true if row and column exists, false otherwise
     */
    public boolean addCell(int ri, int ci, float localCount, 
			   float localWeight, float weight)
    {
	IObject rowObject = rows.getIObjectWithId(ri);
	IObject colObject = cols.getIObjectWithId(ci);
	if(rowObject==null || colObject==null)
	    return false;
	IMatrixCell cell = 
	    new IMatrixCell(rowObject, colObject, localCount, 
			    localWeight, weight);
	if(!addCell(cell))
	    return false;
	//cellMap.put(rowObject, colObject, cell);

	//	maxW = Math.max(maxW,weight);
	return true;
    }



    /**
     * Returns the corresponding group, i.e. a group along the other dimension
     * that contains all IObjects that are associated with the IObject:s
     * of the given group.
     */
    public IObjectGroup constructCorrespondingGroup(IObjectGroup group){
	// Think document-term-matrix

	Vector<IObject> colObjects = group.getOrdered();
	HashSet<IObject> rowObjects = new HashSet<IObject>();
	for(int i=0; i<colObjects.size(); i++){
	    IObject io = colObjects.elementAt(i);

	    Iterator cellIt = getCellsWith(io).iterator();
	    while(cellIt.hasNext()){
		IMatrixCell mc = (IMatrixCell) cellIt.next();
		rowObjects.add(mc.getOtherIObject(io));
	    }
	    /*
	    Vector cells = getCellsWith(io);
	    for(int j=0; j<cells.size(); j++){
		IMatrixCell mc = (IMatrixCell) cells.elementAt(j);
		rowObjects.add(mc.getOtherIObject(io));
	    }
	    */
	}

	IObjectGroup correspondingGroup = 
	    new IObjectGroup("Corresponding Group to "+group.getName());
	Iterator<IObject> it = rowObjects.iterator();
	while(it.hasNext())
	    correspondingGroup.add(it.next());
	
	return correspondingGroup;
    }

    /**
     * Returns a Vector of all the IMatrixCells that 
     * contains the given IObject.
     * Each IMatrixCell is returned in one position only.
     */
    public Vector getCellsWith(IObject co){
	HashSet ret = new HashSet();
	Vector v1 = cellMap.valuesForFirstIndex(co);
	Vector v2 = cellMap.valuesForSecondIndex(co);
	ret.addAll(v1);
	ret.addAll(v2);
	return new Vector(ret);
    }


    public String getNumberOfRowsColumnsCellsString(){
	String rowString = "rows="+rows.numberOfIObjects();
	String colString = "columns="+cols.numberOfIObjects();
	String cells = "cells="+cellMap.size();
	return rowString+","+colString+","+cells;
    }


    public boolean rowIObjectSet(IObjectSet ios){
	if(ios==rows)
	    return true;
	return false;
    }

    public boolean columnIObjectSet(IObjectSet ios){
	if(ios==cols)
	    return true;
	return false;
    }

    /*
    public boolean rowObject(IObject io){
	if(rows.contains(io))
	    return true;
	if(io instanceof IObjectGroup){
	    IObjectGroup iog = (IObjectGroup) io;
	    if(rows.containsIObjectGroup(iog))
		return true;
	}
	if(io instanceof IObjectGrouping){
	    IObjectGrouping ioging = (IObjectGrouping) io;
	    if(rows.containsIObjectGrouping(ioging))
		return true;
	}
	return false;
    }
    */


    /**
     * Returns the total weight for each object along the specified
     * dimension.
     *
     * @param row - true: weight for row objects, 
     *              false: weight for column objects
     * @param localWeight - true: the localWeght,
     *                      false: the weight
     */ 
    public HashMap<IObject, Float> getTotalWeight(boolean row, 
						  boolean localWeight){
	HashMap<IObject, Float> totalLocalWeightMap = 
	    new HashMap<IObject, Float>();
	Vector<IObject> ios = cols.getOrdered();
	if(row)
	    ios = rows.getOrdered();

	for(int i=0; i<ios.size(); i++){
	    IObject io = ios.elementAt(i);
	    Iterator cellIt = getCellsWith(io).iterator();
	    while(cellIt.hasNext()){
		IMatrixCell mc = (IMatrixCell) cellIt.next();
		float mcWeight = mc.getWeight();
		if(localWeight)
		    mcWeight = mc.getLocalWeight();
		IObject io2 = mc.getOtherIObject(io);
		Float weight = totalLocalWeightMap.get(io2);

		//		System.out.println("- "+io2.getString()+": "+weight);

		if(weight!=null)
		    totalLocalWeightMap.put(io2, weight+mcWeight);
		else
		    totalLocalWeightMap.put(io2, mcWeight);
	    }
	}
	return totalLocalWeightMap;
    }




    /**
     * Removes all IObjects that's not in the current IObjectGroupings
     * from the sets and all their IObjectGroupings. Also removes
     * all corresponding IMatrixCells.
     */
    /*
    public void purgeMatrixToCurrentGroupings(){
	// Remove IObjects from Sets
	if(rows!=null)
	    rows.purgeSetToCurrentGrouping();
	if(cols!=null)
	    cols.purgeSetToCurrentGrouping();

	// Remove IMatrix cells that contains IObject:s 
	// that were removed.
	HashSet<IObject> rowSet = new HashSet<IObject>(rows.getOrdered());
	HashSet<IObject> colSet = new HashSet<IObject>(cols.getOrdered());
	Vector cells = getAllCells();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cells.elementAt(i);
	    if(!rowSet.contains(mc.getRowIObject()) ||
	       !colSet.contains(mc.getColumnIObject())){
		removeCell(mc);

		removeRowIObjectGlobalWeight(mc.getRowIObject());
		removeColumnIObjectGlobalWeight(mc.getColumnIObject());
	    }
	}
	resort();
    }
    */


    /**
     * Removes all IObjects that's not in the specified row and colum group.
     * Also removes all corresponding IMatrixCells.
     */
    public void purgeMatrixToGroups(IObjectGroup rowGroup, IObjectGroup columnGroup){
	// Remove IMatrix cells that contains IObject:s 
	// in the Groups.
	HashSet<IObject> rowSet = new HashSet<IObject>(rows.getOrdered());
	HashSet<IObject> colSet = new HashSet<IObject>(cols.getOrdered());
	Vector cells = getAllCells();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cells.elementAt(i);
	    if(!rowGroup.contains(mc.getRowIObject()) ||
	       !columnGroup.contains(mc.getColumnIObject())){
		removeCell(mc);
	    }
	}

	// Remove IObjects from Sets
	if(rows!=null){
	    Vector<IObject> ios = rows.removeObjectsNotIn(rowGroup);
	    for(int i=0; i<ios.size(); i++)
		removeRowIObjectGlobalWeight(ios.get(i));
	}
	if(cols!=null){
	    Vector<IObject> ios = cols.removeObjectsNotIn(columnGroup);
	    for(int i=0; i<ios.size(); i++)
		removeColumnIObjectGlobalWeight(ios.get(i));
	}
	//	resort();
    }





    public void invertMatrix(){
	// Invert IObjectSet:s
	IObjectSet tmp = rows;
	rows=cols;
	cols=tmp;
		
	// Invert all IMatrixCell:s
	cellMap.invert();
	Vector cells = getAllCells();
	//	cellMap = new DoubleHashMap();
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cells.elementAt(i);
	    mc.invert();
	    //	    addCell(mc);
	    /*
	      IObject tmpO = mc.getRowIObject();
	      mc.setRowIObject(mc.getColumnIObject());
	      mc.setColumnIObject(tmpO);
	    */
	}
    }
	
	
    /**
     * Returns the "row" IObjectSet.
     */
    public IObjectSet getRowIObjectSet(){
	return rows;
    }

    /**
     * Returns the "column" IObjectSet.
     */
    public IObjectSet getColumnIObjectSet(){
	return cols;
    }


    /**
     * Sets the "row" IObjectSet. 
     * WARNING. This is usually not a good idea!
     */
    public void setRowIObjectSet(IObjectSet set){
	this.rows=set;
    }

    /**
     * Sets the "column" IObjectSet. 
     * WARNING. This is usually not a good idea!
     */
    public void setColumnIObjectSet(IObjectSet set){
	this.cols=set;
    }


    /**
     * Returns the current IObjectGrouping for the rows.
     */
    /*
    public IObjectGrouping getCurrentRowGrouping(){
	return rows.getCurrentGrouping();
    }
    */

    /**
     * Returns the current IObjectGrouping for the columns.
     */
    /*
    public IObjectGrouping getCurrentColumnGrouping(){
	return cols.getCurrentGrouping();
    }
    */

    /**
     * Returns an IVector with all cells and their weights.
     */
    public IVector<IMatrixCell> getIVectorOfElements(){
	IVector<IMatrixCell> iv = new IVector<IMatrixCell>();
	Vector cells = getAllCells();
	for(int i = 0; i<cells.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cells.get(i);
	    iv.add(mc, mc.getWeight());
	}
	return iv;
    }


    /**
     * Returns the DoubleHashMap containing all 
     * non zero elements of the IMatrix.
     */
    public DoubleHashMap getCellMap(){
	return cellMap;
    }

    /**
     * Returns the DoubleHashMap containing all 
     * non zero elements of the IMatrix.
     */
    public Vector getAllCells(){
	return cellMap.values();
    }

    /**
     * Returns the IMatrixCell for the IObject "row" in the "row" IObjectSet
     * and the IObject "col" in the "column" IObjectSet
     */
    public IMatrixCell getCell(IObject row, IObject col){
	return (IMatrixCell) cellMap.get(row, col);
    }

    /**
     * Removes a IMatrixCell from the IMatrix. Does not remove the 
     * corresponding IObjects from the IObjectSet:s.
     * @param mc An IMatrixCell
     */
    public void removeCell(IMatrixCell mc){
	cellMap.remove(mc.getRowIObject(), mc.getColumnIObject());
	
    }
	
    public void removeCells(IVector<IMatrixCell> ivec){
	Vector<IMatrixCell> vec = ivec.getUnorderedVector();
	for(int i=0; i<vec.size(); i++)
	    removeCell(vec.get(i));
    }


    /**
     * Revmove all IMatrixCell from the IMatrix. Does not remove the 
     * corresponding IObjects from the IObjectSet:s.
     */
    public void removeAllCells(){
	cellMap=new DoubleHashMap();
    }
	
    /**
     * Returns a Dimension with x-size as the number of "columns"
     * and y-size as the number of "rows" of the IMatrix.
     */
    public Dimension getSize(){
	return new Dimension(cols.numberOfIObjects(), rows.numberOfIObjects());
    }

    /**
     * Returns a Dimension with x-size as the number of "columns"
     * and y-size as the number of "rows" of the IMatrix.
     * This is the size of the matrix seen through the current row and
     * column groupings.
     */
    /*
    public Dimension getCurrentSize(){
	return new Dimension(getCurrentColumnGrouping().numberOfIObjects(), 
			     getCurrentRowGrouping().numberOfIObjects());
    }
    */

    /**
     * Resorts the IObjectSet:s "rows" and "cols" according to their
     * current IObjectGroupings.
     */
    /*
    public void resort(){
	//rows.sortAccordingToGrouping(rows.getCurrentGrouping());
	//cols.sortAccordingToGrouping(cols.getCurrentGrouping());
	rows.getCurrentGrouping().sort();
	cols.getCurrentGrouping().sort();
    }
    */


    /**
     * Returns a Vector of the IMatrixCell:s that contains the given IObject
     * AND any of the IObjects in the restrictionGroup.
     */
    public Vector 
	getCellsWithIObjectRestrictedToGroup(IObject io, IObjectGroup restrictionGroup){
	// Frut:
	//     * OBS. This does not work properly if the same IObjectSet is used for
	//     * both rows and columns. Thus to construct a word-word-matrix, for 
	//     * instance, you should have each word as two different IObject:s.
	
	Vector cs = getCellsWith(io);
	/*
	Vector cs=null;
	if(rows.contains(io))
	    cs = cellMap.valuesForFirstIndex(io);
	else if(cols.contains(io))
	    cs = cellMap.valuesForSecondIndex(io);
	if(cs==null)
	    return null;

	    
	// If the entire set is used as restriction
	if(cs.size() == restrictionGroup.numberOfIObjects())
	    return cs;
	*/

	Vector cells = new Vector();
	for(int i=0; i<cs.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cs.elementAt(i);
	    IObject io2 = mc.getOtherIObject(io);
	    if(restrictionGroup.contains(io2))
		cells.add(mc);
	}
	return cells;
    }

    /**
     * Returns a Vector of the IMatrixCells that contains the given IObject.
     * OBS. This does not work properly if the same IObjectSet is used for
     * both rows and columns. Thus to construct a word-word-matrix, for 
     * instance, you should have each word as two different IObject:s.
     */
    /*
    public Vector getCellsWith(IObject co){
	if(rows.contains(co))
	    return cellMap.valuesForFirstIndex(co);
	else if(cols.contains(co))
	    return cellMap.valuesForSecondIndex(co);
	return null;
    }
    */

    public HashSet getRowIMatrixCellsForColumnObject(IObject colObject){
	return new HashSet(cellMap.valuesForSecondIndex(colObject));
    }
	
    public HashSet getColumnIMatrixCellsForRowObject(IObject rowObject){
	Collection cells = cellMap.valuesForFirstIndex(rowObject);
	if(cells==null)
	    return null;
	return new HashSet(cells);
    }
	
    /**
     * Returns all IObject:s the specified IObject share a IMatrixCell
     * with.
     */
    public HashSet<IObject> getOtherIObjectsForIObject(IObject io){
	HashSet<IObject> ret = new HashSet<IObject>();
	Vector cells = getCellsWith(io);
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cells.elementAt(i);
	    ret.add(mc.getOtherIObject(io));
	}
	return ret;
    }


    /*
    public HashSet<IObject> getIObjectsForIObject(IObject io){
	HashSet<IObject> ret = new HashSet<IObject>();
	Vector cells = getCellsWith(io);
	for(int i=0; i<cells.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cells.elementAt(i);
	    IObject rio = mc.getRowIObject();
	    if(rio==io)
		rio = mc.getColumnIObject();
	    ret.add(rio);
	}
	return ret;
    }
    */


    public void saveIMatrix(File file){
	IMatrixIO iomatrix = new IMatrixIO();
	iomatrix.save(this, file);
    }
	
    public void saveIObjectGroupingFile(IObjectGrouping grouping, File file){
	IObjectGroupingIO iogrouping = new IObjectGroupingIO();
	iogrouping.save(grouping, this, file);
    }


    public double avgColsInRows(){
		
	Vector<IObject> vecio = rows.getOrdered();
	int sum = 0;
	for(int i = 0; i < vecio.size(); i++){
	    sum += cellMap.valuesForFirstIndex(vecio.get(i)).size();
	}
		
	return Round.roundDecimals((double)sum/rows.numberOfIObjects(), 4);
    }
	
    public double avgRowsInCols(){
		
	Vector<IObject> vecio = cols.getOrdered();
	int sum = 0;
	for(int i = 0; i < vecio.size(); i++){
	    sum += cellMap.valuesForSecondIndex(vecio.get(i)).size();
	}
		
	return Round.roundDecimals((double)sum/cols.numberOfIObjects(), 4);
    }

    public void setRowIObjectGlobalWeight(IObject io, float globalWeight){
	rowIObjectGlobalWeightMap.put(io, globalWeight);
    }

    public void setColumnIObjectGlobalWeight(IObject io, float globalWeight){
	colIObjectGlobalWeightMap.put(io, globalWeight);
    }


    public void removeRowIObjectGlobalWeight(IObject io){
	rowIObjectGlobalWeightMap.remove(io);
    }

    public void removeColumnIObjectGlobalWeight(IObject io){
	colIObjectGlobalWeightMap.remove(io);
    }


    /**
     * Returns 0 if the IObject has not been weighted.
     */
    public float getRowIObjectGlobalWeight(IObject io){
	Float weight = rowIObjectGlobalWeightMap.get(io);
	if(weight!=null)
	    return weight;
	return 0;
    }

    /**
     * Returns 0 if the IObject has not been weighted.
     */
    public float getColumnIObjectGlobalWeight(IObject io){
	Float weight = colIObjectGlobalWeightMap.get(io);
	if(weight!=null)
	    return weight;
	return 0;
    }


}
