package infomat.vectorspace.similarity;

import infomat.algorithms.grouping.NeedsIMatrix;
import infomat.algorithms.grouping.NeedsIMatrixDimension;
import infomat.vectorspace.*;
import infomat.vectorspace.vector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;

/**
 * Cartesian Similarity is the inverse of the Cartesian distance:
 * sim = 1/dist. If the distance is zero the similarity is set
 * to infinity.
 *
 * @version          090105
 * @author           Magnus Rosell
 */
public class SparseCartesianSimilarity extends SparseISimilarity
    implements NeedsIMatrix, NeedsIMatrixDimension{

    private IMatrix matrix=null;
    private boolean row=true;

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("Cartesian Similarity Properties", CurrentClassGetter.getClassName());
		
	/*
	try{
	    
	    Property p = new Property("Normalize", false, "Wether normalization is used. This can also be done through the Weighter.");
	    PropertyValueDescription pvd = 
		new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	*/

	return properties;
    }

    public String getName(){
    	return "Cartesian Similarity";
    }

    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
	/*
	if(name.compareTo("Normalize")==0)
	    normalize = (Boolean) getPropertyValue("Normalize");
	*/
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }


    //    private Boolean normalize = false;

    public SparseCartesianSimilarity(){
	super();
	//	this.normalize = (Boolean) getPropertyValue("Normalize");
    }

    public SparseCartesianSimilarity(SparseCartesianSimilarity sim){
	super(sim);
	//	this.normalize = (Boolean) getPropertyValue("Normalize");
	this.matrix=sim.matrix;
	this.row=sim.row;
    }

    public SparseCartesianSimilarity(IMatrix matrix, boolean row){
	super();
	this.matrix=matrix;
	this.row=row;
	//	this.normalize = (Boolean) getPropertyValue("Normalize");
    }

    public SparseCartesianSimilarity(IMatrix matrix, boolean row, 
				     mro.util.properties.Properties properties){
	super();
	this.matrix=matrix;
	this.row=row;
    	this.properties=properties;
	//	this.normalize = (Boolean) getPropertyValue("Normalize");
    }



    /**
     * Calculates the similarity between two IObject:s.
     * Extracts the representation from the matrix.
     */
    public float sim(IObject io1, IObject io2){

	IVector<IVectorIndex> vec1 = getRepresentation(io1);
	IVector<IVectorIndex> vec2 = getRepresentation(io2);

	return sim(vec1, vec2);
    }

    /**
     * Calculates the similarity between IObject and a IObjectGroup.
     * Extracts the representation from the matrix.
     * Uses the group representation for IObjectGroup:s of all kinds.
     * This takes more time to extract. Therefore it might be better
     * to use the #getGroupRepresentation-method instead...
     */
    public float sim(IObject io, IObjectGroup iog){

	IVector<IVectorIndex> ioVec = getRepresentation(io);
	IVector<IVectorIndex> groupVec = getRepresentation(iog);

	return sim(ioVec, groupVec);
    }

    /**
     * Calculates the similarity between two IObjectGroup:s.
     * Extracts the representation from the matrix.
     * Uses the group representation for IObjectGroup:s of all kinds.
     * This takes more time to extract. Therefore it might be better
     * to use the #getGroupRepresentation-method instead...
     */
    public float sim(IObjectGroup iog1, IObjectGroup iog2){

	IVector<IVectorIndex> vec1 = getGroupRepresentation(iog1);
	IVector<IVectorIndex> vec2 = getGroupRepresentation(iog2);

	return sim(vec1, vec2);
    }


	
    /**
     * Calculates the similarity between two representations.
     */
    public <U extends IVectorIndex, V extends IVectorIndex> float sim(IVector<U> vec1, IVector<V> vec2){

	Float dist = IVector.cartesianDistance(vec1, vec2);
	if(dist==null)
	    return Float.NEGATIVE_INFINITY;
	if(dist==0)
	    return Float.POSITIVE_INFINITY;

	return (float) (1.0/dist);
    }


    /**
     * Calculates the similarity between two representations,
     * using the distance from origo for the second.
     */
    public <U extends IVectorIndex, V extends IVectorIndex> float sim(IVector<U> vec1, IVector<V> vec2, double sqDist){

	Float dist = IVector.cartesianDistance(vec1, vec2, sqDist);
	if(dist==null)
	    return Float.NEGATIVE_INFINITY;
	if(dist==0)
	    return Float.POSITIVE_INFINITY;

	return (float) (1.0/dist);
    }


    public <V extends IVectorIndex> double 
		      cartesianSquaredDistanceFromOrigo(IVector<V> vec1){
	return IVector.cartesianSquaredDistanceFromOrigo(vec1);
    }


    /**
     * Returns the representation in the form of a IVector<IObject>.
     */
    public IVector<IVectorIndex> getRepresentation(IObject io){

	IVector<IVectorIndex> representation = new IVector<IVectorIndex>();
	HashSet features = null;
	if(getIMatrixDimension())
	    features = getIMatrix().getColumnIMatrixCellsForRowObject(io);
	else
	    features = getIMatrix().getRowIMatrixCellsForColumnObject(io);

	if(features!=null){
	    Iterator it = features.iterator();
	    while(it.hasNext()){
		IMatrixCell sc = (IMatrixCell) it.next();
		IObject feature = null;
		if(getIMatrixDimension())
		    feature = sc.getColumnIObject();
		else
		    feature = sc.getRowIObject();
		representation.put(feature, sc.getWeight());
	    }
	}
	return representation;
    }



	

    public IVector<IVectorIndex> getGroupRepresentation(IObjectGroup iog){
	IVector<IVectorIndex> centroid = new IVector<IVectorIndex>();

	// All IObject:s in the IObjectGroup
	Vector<IObject> ordered = iog.getOrdered();
	for(int i=0; i<ordered.size(); i++){
	    IObject io = ordered.elementAt(i);

	    // All IObjects along the other dimension of the IMatrix
	    HashSet features = null;
	    if(getIMatrixDimension())
		features = getIMatrix().getColumnIMatrixCellsForRowObject(io);
	    else
		features = getIMatrix().getRowIMatrixCellsForColumnObject(io);
	    
	    if(features!=null){
		Iterator it = features.iterator();
		while(it.hasNext()){
		    IMatrixCell sc = (IMatrixCell) it.next();

		    IObject feature = null;
		    if(getIMatrixDimension())
			feature = sc.getColumnIObject();
		    else
			feature = sc.getRowIObject();

		    centroid.add(feature, sc.getWeight());
		}
	    }
	}

	// Average weight
	centroid.multiply(((float) 1.0/iog.numberOfIObjects()));

	return centroid;
    }




    public void setIMatrix(IMatrix matrix){
	this.matrix=matrix;
    }

    public IMatrix getIMatrix(){
	return matrix;
    }

    public void setIMatrixDimension(boolean row){
	this.row=row;
    }

    public boolean getIMatrixDimension(){
	return this.row;
    }

}



